// MilkshapeModel.c

#include <windows.h>		// Header File For Windows
#include <stdio.h>
#include <gl\gl.h>			// Header File For The OpenGL32 Library

#include "MilkshapeModel.h"

BOOL Model_LoadModelData(MODEL * pM, const char *filename )
{
	FILE * file = fopen( filename, "rb" );
	if ( !file )
		return FALSE;	// "Couldn't open the model file."

	fseek(file, 0, SEEK_END );
	long fileSize = ftell(file);
	fseek(file, 0, SEEK_SET );

	byte *pBuffer = malloc(fileSize);
	fread(pBuffer, fileSize, 1, file );
	fclose(file);

	const byte *pPtr = pBuffer;
	MS3DHEADER *pHeader = ( MS3DHEADER* )pPtr;
	pPtr += sizeof( MS3DHEADER );

	if ( strncmp( pHeader->m_ID, "MS3D000000", 10 ) != 0 )
		return FALSE; // "Not a valid Milkshape3D model file."

	if ( pHeader->m_version < 3 || pHeader->m_version > 4 )
		return FALSE; // "Unhandled file version. Only Milkshape3D Version 1.3 and 1.4 is supported." );

	int nVertices = *( word* )pPtr;
	pM->m_numVertices = nVertices;
	pM->m_pVertices   = malloc(sizeof(VERTEX) * nVertices);
	pPtr += sizeof( word );

	int i;
	for ( i = 0; i < nVertices; i++ )
	{
		MS3DVERTEX *pVertex = ( MS3DVERTEX* )pPtr;
		pM->m_pVertices[i].m_boneID = pVertex->m_boneID;
		memcpy( pM->m_pVertices[i].m_location, pVertex->m_vertex, sizeof( float )*3 );
		pPtr += sizeof( MS3DVERTEX );
	}

	int nTriangles = *( word* )pPtr;
	pM->m_numTriangles = nTriangles;
	pM->m_pTriangles   = malloc(sizeof(TRIANGLE) * nTriangles);
	pPtr += sizeof( word );
	int vertexIndices[3];
	float t[3];

	for ( i = 0; i < nTriangles; i++ )
	{
		MS3DTRIANGLE * pTriangle = ( MS3DTRIANGLE* )pPtr;
	   	vertexIndices[0] = pTriangle->m_vertexIndices[0];
		vertexIndices[1] = pTriangle->m_vertexIndices[1];
		vertexIndices[2] = pTriangle->m_vertexIndices[2];
		t[0] = 1.0f - pTriangle->m_t[0];
		t[1] = 1.0f - pTriangle->m_t[1];
		t[2] = 1.0f - pTriangle->m_t[2];
		memcpy( pM->m_pTriangles[i].m_vertexNormals, pTriangle->m_vertexNormals, sizeof( float )*3*3 );
		memcpy( pM->m_pTriangles[i].m_s, pTriangle->m_s, sizeof( float )*3 );
		memcpy( pM->m_pTriangles[i].m_t, t, sizeof( float )*3 );
		memcpy( pM->m_pTriangles[i].m_vertexIndices, vertexIndices, sizeof( int )*3 );
		pPtr += sizeof( MS3DTRIANGLE );
	}

	int nGroups = *( word* )pPtr;
	pM->m_numMeshes = nGroups;
	pM->m_pMeshes   = malloc(sizeof(MESH) * nGroups);
	pPtr += sizeof( word );
	for ( i = 0; i < nGroups; i++ )
	{
		pPtr += sizeof( byte );	// flags
		pPtr += 32;				// name

		word nTriangles = *( word* )pPtr;
		pPtr += sizeof( word );
		int * pTriangleIndices = malloc(sizeof(int) * nTriangles);
		for ( int j = 0; j < nTriangles; j++ )
		{
			pTriangleIndices[j] = *( word* )pPtr;
			pPtr += sizeof( word );
		}

		char materialIndex = *( char* )pPtr;
		pPtr += sizeof( char );

		pM->m_pMeshes[i].m_materialIndex = materialIndex;
		pM->m_pMeshes[i].m_numTriangles = nTriangles;
		pM->m_pMeshes[i].m_pTriangleIndices = pTriangleIndices;
	}

	int nMaterials = *( word* )pPtr;
	pM->m_numMaterials = nMaterials;
	pM->m_pMaterials   = malloc(sizeof(MATERIAL) * nMaterials);
	pPtr += sizeof( word );
	for ( i = 0; i < nMaterials; i++ )
	{
		MS3DMATERIAL *pMaterial = ( MS3DMATERIAL* )pPtr;
		memcpy( pM->m_pMaterials[i].m_ambient, pMaterial->m_ambient, sizeof( float )*4 );
		memcpy( pM->m_pMaterials[i].m_diffuse, pMaterial->m_diffuse, sizeof( float )*4 );
		memcpy( pM->m_pMaterials[i].m_specular, pMaterial->m_specular, sizeof( float )*4 );
		memcpy( pM->m_pMaterials[i].m_emissive, pMaterial->m_emissive, sizeof( float )*4 );
		pM->m_pMaterials[i].m_shininess = pMaterial->m_shininess;
		pM->m_pMaterials[i].m_pTextureFilename = malloc(strlen( pMaterial->m_texture )+1);
		strcpy( pM->m_pMaterials[i].m_pTextureFilename, pMaterial->m_texture );
		pPtr += sizeof( MS3DMATERIAL );
	}

	Model_ReloadTextures(pM);

	return TRUE;
}
