// MilkshapeModel.h

#ifndef _MILKSHAPEMODEL_H
#define _MILKSHAPEMODEL_H

#include "Model.h"

#pragma pack( push, 1 )

typedef unsigned char byte;
typedef unsigned short word;

// File header
typedef struct MS3DHeader
{
	char m_ID[10];
	int m_version;
}MS3DHEADER;

// Vertex information
typedef struct MS3DVertex
{
	byte m_flags;
	float m_vertex[3];
	char m_boneID;
	byte m_refCount;
}MS3DVERTEX;

// Triangle information
typedef struct MS3DTriangle
{
	word m_flags;
	word m_vertexIndices[3];
	float m_vertexNormals[3][3];
	float m_s[3], m_t[3];
	byte m_smoothingGroup;
	byte m_groupIndex;
}MS3DTRIANGLE;

// Material information
typedef struct MS3DMaterial
{
    char m_name[32];
    float m_ambient[4];
    float m_diffuse[4];
    float m_specular[4];
    float m_emissive[4];
    float m_shininess;	// 0.0f - 128.0f
    float m_transparency;	// 0.0f - 1.0f
    byte m_mode;	// 0, 1, 2 is unused now
    char m_texture[128];
    char m_alphamap[128];
}MS3DMATERIAL;

// Keyframe data
typedef struct MS3DKeyframe
{
	int m_jointIndex;
	float m_time;		// millisecs
	float m_parameter[3];
}MS3DKEYFRAME;

#pragma pack( pop )

//Load the model data into the private variables.
BOOL Model_LoadModelData(MODEL * pM, const char *filename );

#endif // _MILKSHAPEMODEL_H
