// model.c

#include <windows.h>		// Header File For Windows
#include <gl\gl.h>			// Header File For The OpenGL32 Library

#include "Model.h"

extern GLuint LoadGLTexture( const char *filename );	// Load Bitmaps And Convert To Textures

void Model_Init(MODEL * pM)
{
	pM->m_numMeshes 	= 0;
	pM->m_pMeshes 		= NULL;
	pM->m_numMaterials  = 0;
	pM->m_pMaterials 	= NULL;
	pM->m_numTriangles  = 0;
	pM->m_pTriangles 	= NULL;
	pM->m_numVertices   = 0;
	pM->m_pVertices 	= NULL;
}

void Model_Delete(MODEL * pM)
{
	int i;
	for ( i = 0; i < pM->m_numMeshes; i++ )
		if(pM->m_pMeshes[i].m_pTriangleIndices) free(pM->m_pMeshes[i].m_pTriangleIndices);
	for ( i = 0; i < pM->m_numMaterials; i++ )
		if(pM->m_pMaterials[i].m_pTextureFilename) free(pM->m_pMaterials[i].m_pTextureFilename);

	pM->m_numMeshes = 0;
	if ( pM->m_pMeshes != NULL )
	{
		if(pM->m_pMeshes) free(pM->m_pMeshes);
		pM->m_pMeshes = NULL;
	}

	pM->m_numMaterials = 0;
	if ( pM->m_pMaterials != NULL )
	{
		if(pM->m_pMaterials) free(pM->m_pMaterials);
		pM->m_pMaterials = NULL;
	}

	pM->m_numTriangles = 0;
	if ( pM->m_pTriangles != NULL )
	{
		if(pM->m_pTriangles) free(pM->m_pTriangles);
		pM->m_pTriangles = NULL;
	}

	pM->m_numVertices = 0;
	if ( pM->m_pVertices != NULL )
	{
		if(pM->m_pVertices) free(pM->m_pVertices);
		pM->m_pVertices = NULL;
	}
}

void Model_Draw(MODEL * pM)
{
	GLboolean texEnabled = glIsEnabled( GL_TEXTURE_2D );

	// Draw by group
	for ( int i = 0; i < pM->m_numMeshes; i++ )
	{
		int materialIndex = pM->m_pMeshes[i].m_materialIndex;
		if ( materialIndex >= 0 )
		{
			glMaterialfv( GL_FRONT, GL_AMBIENT, pM->m_pMaterials[materialIndex].m_ambient );
			glMaterialfv( GL_FRONT, GL_DIFFUSE, pM->m_pMaterials[materialIndex].m_diffuse );
			glMaterialfv( GL_FRONT, GL_SPECULAR, pM->m_pMaterials[materialIndex].m_specular );
			glMaterialfv( GL_FRONT, GL_EMISSION, pM->m_pMaterials[materialIndex].m_emissive );
			glMaterialf( GL_FRONT, GL_SHININESS, pM->m_pMaterials[materialIndex].m_shininess );

			if ( pM->m_pMaterials[materialIndex].m_texture > 0 )
			{
				glBindTexture( GL_TEXTURE_2D, pM->m_pMaterials[materialIndex].m_texture );
				glEnable( GL_TEXTURE_2D );
			}
			else
				glDisable( GL_TEXTURE_2D );
		}
		else
		{
			// Material properties?
			glDisable( GL_TEXTURE_2D );
		}

		glBegin( GL_TRIANGLES );
		{
			for ( int j = 0; j < pM->m_pMeshes[i].m_numTriangles; j++ )
			{
				int triangleIndex = pM->m_pMeshes[i].m_pTriangleIndices[j];
				const TRIANGLE * pTri = &pM->m_pTriangles[triangleIndex];

				for ( int k = 0; k < 3; k++ )
				{
					int index = pTri->m_vertexIndices[k];

					glNormal3fv( pTri->m_vertexNormals[k] );
					glTexCoord2f( pTri->m_s[k], pTri->m_t[k] );
					glVertex3fv( pM->m_pVertices[index].m_location );
				}
			}
		}
		glEnd();
	}

	if ( texEnabled )
		glEnable( GL_TEXTURE_2D );
	else
		glDisable( GL_TEXTURE_2D );
}

void Model_ReloadTextures(MODEL * pM)
{
	for ( int i = 0; i < pM->m_numMaterials; i++ )
		if ( strlen( pM->m_pMaterials[i].m_pTextureFilename ) > 0 )
			pM->m_pMaterials[i].m_texture = LoadGLTexture( pM->m_pMaterials[i].m_pTextureFilename );
		else
			pM->m_pMaterials[i].m_texture = 0;
}


