// Model.h

#ifndef MODEL_H
#define MODEL_H

typedef struct Mesh
{
	int   m_materialIndex;
	int   m_numTriangles;
	int * m_pTriangleIndices;
}MESH;

//	Material properties
typedef struct Material
{
	float m_ambient[4];
	float m_diffuse[4];
	float m_specular[4];
	float m_emissive[4];
	float m_shininess;
	GLuint m_texture;
	char *m_pTextureFilename;
}MATERIAL;

//	Triangle structure
typedef struct Triangle
{
	float m_vertexNormals[3][3];
	float m_s[3];
	float m_t[3];
	int   m_vertexIndices[3];
}TRIANGLE;

//	Vertex structure
typedef struct _Vertex
{
	// for skeletal animation
	char  m_boneID;
	float m_location[3];
}VERTEX;

typedef struct _Model
{
	int m_numMeshes;
	MESH * m_pMeshes;

	//	Materials used
	int m_numMaterials;
	MATERIAL *m_pMaterials;

	//	Triangles used
	int m_numTriangles;
	TRIANGLE *m_pTriangles;

	//	Vertices Used
	int m_numVertices;
	VERTEX * m_pVertices;
}MODEL;

BOOL Model_LoadModelData(MODEL * pM, const char *filename );
void Model_Draw(MODEL * pM);
void Model_ReloadTextures(MODEL * pM);
void Model_Init(MODEL * pModel);
void Model_Delete(MODEL * pM);

#endif // ndef MODEL_H
